////////////////////////////////////////
//  File      : axkb.cpp         //
//  Written by: disels@gmail.com      //
//  Copyright : GPL                   //
////////////////////////////////////////

#include "axkb.h"

AXKB::AXKB(int &argc, char **argv) : QApplication(argc, argv)
{
    keys = new XKeyboard ();
    initKeyCodeMap();
}


void AXKB::set_event_names()
{
    event_names.insert(0, "");
    event_names.insert(1, "");
    event_names.insert(2, "KeyPress");
    event_names.insert(3, "KeyRelease");
    event_names.insert(4, "ButtonPress");
    event_names.insert(5, "ButtonRelease");
    event_names.insert(6, "MotionNotify");
    event_names.insert(7, "EnterNotify");
    event_names.insert(8, "LeaveNotify");
    event_names.insert(9, "FocusIn");
    event_names.insert(10, "FocusOut");
    event_names.insert(11, "KeymapNotify");
    event_names.insert(12, "Expose");
    event_names.insert(13, "GraphicsExpose");
    event_names.insert(14, "NoExpose");
    event_names.insert(15, "VisibilityNotify");
    event_names.insert(16, "CreateNotify");
    event_names.insert(17, "DestroyNotify");
    event_names.insert(18, "UnmapNotify");
    event_names.insert(19, "MapNotify");
    event_names.insert(20, "MapRequest");
    event_names.insert(21, "ReparentNotify");
    event_names.insert(22, "ConfigureNotify");
    event_names.insert(23, "ConfigureRequest");
    event_names.insert(24, "GravityNotify");
    event_names.insert(25, "ResizeRequest");
    event_names.insert(26, "CirculateNotify");
    event_names.insert(27, "CirculateRequest");
    event_names.insert(28, "PropertyNotify");
    event_names.insert(29, "SelectionClear");
    event_names.insert(30, "SelectionRequest");
    event_names.insert(31, "SelectionNotify");
    event_names.insert(32, "ColormapNotify");
    event_names.insert(33, "ClientMessage");
    event_names.insert(34, "MappingNotify");
}

void AXKB::setStartup()
{
    firstStart();
    QSettings * antico = new QSettings(QDir::homePath() + "/.config/axkb.cfg", QSettings::IniFormat, this);
    antico->beginGroup("Style");
    map_path = antico->value("path").toString()+"/language/";
    antico->endGroup(); //Style
    xkbConf = X11tools::loadXKBconf();
    trayIcon = new QSystemTrayIcon(this);
    load_rules();

    if (xkbConf->status!=DONT_USE_XKB)
    {

        qDebug()<<"XKB status : " <<xkbConf->status;
        if (xkbConf->status==USE_XKB)
            set_xkb();
        init();
        qDebug()<<"Show single" << xkbConf->showSingle;
        if (groupeName.count()>1 || xkbConf->showSingle)
        {
            createMenu();
            draw_icon();

        }
        else
        {
             configure();

        }
    }
    else
    {
        configure();
     }
}

AXKB::~AXKB()
{

}

bool AXKB::firstStart()
{
    if (!QFile::exists(QDir::homePath() + "/.config/axkb.cfg"))
    {
        QSettings * antico = new QSettings(QDir::homePath() + "/.config/axkb.cfg", QSettings::IniFormat, this);
        QString themePath;
        QDir themeDir("/usr/share/axkb/theme/default");
        if (themeDir.exists())
           themePath=themeDir.path();
        else
           themePath =   QCoreApplication::applicationDirPath()+"/theme/default";;
        antico->beginGroup("Style");
        map_path = themePath;
        antico->setValue("path",map_path);
        antico->endGroup(); //Style
        return true;
     }
    return false;
}

bool AXKB::x11EventFilter(XEvent *event)
{
//   qDebug()<<"Active windows id :" << getActiveWindowId();

//   qDebug()<<"Active vindows name :" << getActiveWindowAppName();
   checkLayoutChenge();///
   // qDebug()<<"Current event"<<event->type;
   switch (event->type)
    {
       case KeyPress:
       case KeyRelease:
        {
          qDebug()<<"Key pressed" << event->xkey.keycode;
           if( 127 == event->xkey.keycode)
               shotcutConvert();
        }
        break;
        default:
            keys->processEvent(event);
            return false;
    }
   return false;
}

void AXKB::init()
{
       set_event_names();
       groupeName.clear();
       if (xkbConf->status==DONT_USE_XKB)
           return;
       keys->getGroupNames(groupeName);
       currentGroup = keys->getGroupNo();
       if (currentGroup<groupeName.size()-1)
       nextGroupe = currentGroup+1;
       else if (currentGroup == groupeName.size()-1)
           nextGroupe = 0;
       clipboard=  QApplication::clipboard();
       connect(clipboard,SIGNAL(selectionChanged()),SLOT(showClipboard()));
       connect(keys,SIGNAL(groupChanged(int)),this,SLOT(groupChange(int)));
       connect(keys,SIGNAL(layoutChanged()),this,SLOT(layoutChange()));


 }

int AXKB::getLayoutNumber()
{
    XkbStateRec rec;
    XkbGetState(QX11Info::display(), XkbUseCoreKbd, &rec);
    return (int)rec.group;
}

void AXKB::initKeyCodeMap()
{
        /* pc104 code lowwer [  ]  ;   '  ,  .
       pc104 code upper  {  }  :   "  <  >
       code              34 35 47 48 59  60
    */
    lowwerSymbol.insert("[",34);
    lowwerSymbol.insert("]",35);
    lowwerSymbol.insert(";",47);
    lowwerSymbol.insert("'",48);
    lowwerSymbol.insert("`",49);
    lowwerSymbol.insert(",",59);
    lowwerSymbol.insert(".",60);
    upperSymbol.insert("/",62);
    upperSymbol.insert("{",34);
    upperSymbol.insert("}",35);
    upperSymbol.insert(":",47);
    upperSymbol.insert("\"",48);
    upperSymbol.insert("~",49);
    upperSymbol.insert("<",59);
    upperSymbol.insert(">",60);
    upperSymbol.insert("?",62);
}

Window AXKB::getActiveWindowId()
{
    Display* disp = QX11Info::display();
    Window focus_return;
    int revert_to_return;
    XGetInputFocus(disp, &focus_return, &revert_to_return); // get input focus
    return focus_return;
}

QString AXKB::getActiveWindowAppName()
{
   char **window_name_return;
   window_name_return=(char**) malloc(50);
   XFetchName(QX11Info::display(), getActiveWindowId(), window_name_return);
   QString win_name = QString(*window_name_return);
   QStringList tmp = win_name.split("-");
   if (tmp.count()>1) win_name= tmp[tmp.count()-1];
   tmp = win_name.split(":");
   if (tmp.count()>1) win_name= tmp[tmp.count()-1];
   if (win_name.isEmpty() || win_name.isNull())  win_name="NONE";
   win_name.trimmed();
   return win_name;
}

void AXKB::convertString(QString content)
{
    Display* disp = QX11Info::display();
    XkbStateRec rec;
    XkbGetState(disp, XkbUseCoreKbd, &rec);
    int layotNumber = rec.group;
    XKeyEvent xev; // create key event sdtruct
    xev.window = getActiveWindowId();
    xev.serial       = 0;
    xev.send_event   = 0;
    xev.display      = disp;
    xev.root         = None;
    xev.subwindow    = None;
    xev.time         = 0;
    xev.same_screen  = 1;
    xev.state        =0;
    xev.x            = 1;
    xev.y            = 1;
    xev.x_root       = 1;
    xev.y_root       = 1;

       //set state  (2<<12)*layotNumber if louwer case and  (2<<12)*layotNumber+1 if Upper
       // тевт еуые у
 for (int ii=0;ii<content.count();ii++)
    {
       // check space in input string
       if (content[ii].isSpace())
          xev.keycode = 65;
       else
       {
         xev.keycode = lowwerSymbol.value(QString(content[ii]),255);
         xev.state =(2<<12)*layotNumber;
         if  (xev.keycode==255)
         {  xev.keycode = upperSymbol.value(QString(content[ii]),255);
            xev.state =(2<<12)*layotNumber+1;
          }
         if  (xev.keycode==255)
         {
             if (content[ii].isUpper())
                  xev.state =(2<<12)*layotNumber+1;
             else
                  xev.state =(2<<12)*layotNumber;
             char *symbol;
             symbol=(char*) malloc(10);
             strcpy(symbol,QString(content[ii]).toLocal8Bit().data());
             xev.keycode =  XKeysymToKeycode(disp,XStringToKeysym(symbol));
             qDebug()<<XStringToKeysym(symbol);
             if (NoSymbol==xev.keycode || 204== xev.keycode)
             {   xev.keycode = XKeysymToKeycode(disp,X11tools::unicodeToKeysys(content[ii]));
             }
            qDebug()<< xev.keycode;
           }
       }

       xev.type         = KeyPress;
       (void) XSendEvent(disp, xev.window, True, KeyPressMask,(XEvent *)&xev);
       XFlush(disp);
       xev.type         = KeyRelease;
       (void) XSendEvent(disp, xev.window, True, KeyReleaseMask,(XEvent *)&xev);
       XFlush(disp);
    }
}

void AXKB::showClipboard()
{
}

void AXKB::shotcutConvert()
{
    QString buf = clipboard->text(QClipboard::Selection);
    clipboard->clear();
    convertString(buf);
}

void AXKB::draw_icon()
{
   if (groupeName.count()==0) return;
    qDebug()<<"Set icons";
    if (xkbConf->status==DONT_USE_XKB)
    {  exit();
    }
   qDebug()<<"Set menu";
   trayIcon->setContextMenu(contextMenu);
   connect(trayIcon, SIGNAL(activated(QSystemTrayIcon::ActivationReason)), this, SLOT(trayClicked(QSystemTrayIcon::ActivationReason)));
   qDebug()<<"Current groupe"<<currentGroup;
   qDebug()<<"Avaliblegroupe"<<groupeName;
   trayIcon->setToolTip(groupeName[currentGroup]);
   qDebug()<<map_path + rule->layouts.key(groupeName[currentGroup])+".png";

   qDebug()<<xkbConf->showFlag;
     if (xkbConf->showFlag && QFile::exists( map_path +"/"+ rule->layouts.key(groupeName[currentGroup])+".png"))
     {
          trayIcon-> setIcon(QIcon(map_path+"/"+rule->layouts.key(groupeName[currentGroup])+".png"));

     }
     else
     {
        QString layout=rule->layouts.key(groupeName[currentGroup]);
        QFont font("Helvetica [Cronyx]",15);
        font.setBold(true);
        font.setLetterSpacing(QFont::PercentageSpacing,120);
        QPixmap pix(layout.length()*20,28);
        QPainter painter;
        painter.begin(&pix);
        painter.setFont(font);
        painter.fillRect(pix.rect(), Qt::darkGray);
        painter.setPen(Qt::white);
        painter.drawText(pix.rect(),Qt::AlignVCenter|Qt::AlignCenter,layout);
        painter.end();
        trayIcon-> setIcon(QIcon(pix));
     }
    if (xkbConf->status>1)
     {
        trayIcon->setContextMenu(contextMenu);
    }
    trayIcon->show();
}

void AXKB::trayClicked(QSystemTrayIcon::ActivationReason reason)
{

     switch (reason) {
     case QSystemTrayIcon::Trigger:
         setNextGroupe();
         break;
     case QSystemTrayIcon::DoubleClick:
         break;
     case QSystemTrayIcon::MiddleClick:
         setPrevGroupe();
         break;
     default:
         ;
     }
}

void AXKB::layoutChange()
{
    currentGroup=keys->getGroupNo();
    setLanguageMap(currentGroup);
    reconfigure();
}

void AXKB::groupChange(int index)
{
    if (xkbConf->status==DONT_USE_XKB)
      return;
    currentGroup=index;
    if (currentGroup<groupeName.size()-1)
       nextGroupe = currentGroup+1;
    else if (currentGroup == groupeName.size()-1)
       nextGroupe = 0;
    draw_icon();
    setLanguageMap(index);
  }

int AXKB::setKeyLayout(QString keyConf)
{
    QStringList argument = keyConf.split(" ");
    qDebug()<<"setxkbmap argumetns : "<< argument;
    int result = QProcess::execute("setxkbmap",argument);
    qDebug()<<" setxkbmap result : " << result;
    return result;
}

void AXKB::setNextGroupe()
{
     keys->setGroupNo(nextGroupe);
}

void AXKB::setPrevGroupe()
{
     nextGroupe=currentGroup;
     if (currentGroup-1<0)
        currentGroup = groupeName.size()-1;
     else
        currentGroup--;
     keys->setGroupNo(currentGroup);
}

void AXKB::createMenu()
{
    qDebug()<<"Create menu" << groupeName;
    contextMenu = new QMenu();
    qDebug()<<"Avalible groups" << groupeName;
    for (int index=0;index<groupeName.size();index++)
       {
        QAction *act = new QAction(groupeName[index],this) ;
        act->setIcon(QIcon(map_path + rule->layouts.key(groupeName[index])+".png"));
        act->setData(groupeName[index]);
        contextMenu->addAction(act);
        }
    contextMenu->addSeparator();
    QAction *config = new QAction(tr("Configure"),this) ;
    config->setData("configure");
    contextMenu->addAction(config);
    contextMenu->addSeparator();
    QAction *about_qt = new QAction(tr("About Qt"),this) ;
    about_qt->setData("about_qt");
    contextMenu->addAction(about_qt);
    QAction *about_axkb = new QAction(tr("About AXKB"),this) ;
    about_axkb->setData("about_axkb");
    contextMenu->addAction(about_axkb);
    contextMenu->addSeparator();
    QAction *axkbExit = new QAction(tr("Exit"),this) ;
    axkbExit->setData("exit");
    contextMenu->addAction(axkbExit);
    connect(contextMenu,SIGNAL(triggered(QAction*)),SLOT(actionsActivate(QAction*)));
}

void  AXKB::reconfigure()
{
    QStringList tmpGrName;
    keys->getGroupNames(tmpGrName);

    XKBConf* newConf = X11tools::loadXKBconf();
    if (tmpGrName!=groupeName || newConf->layouts != xkbConf->layouts || newConf->showFlag != xkbConf->showFlag || newConf->showSingle != xkbConf->showSingle || newConf->status != xkbConf->status)
    {
        groupeName=tmpGrName;
        xkbConf=newConf;
        init();
        createMenu();
        draw_icon();
    }
}

void  AXKB::set_xkb()
{
    if (xkbConf->status==DONT_USE_XKB)
           return;
    model=xkbConf->model;
    for (int i=0;i<xkbConf->layouts.count();i++)
    {
        layout+=xkbConf->layouts[i].layout;
        variant+=xkbConf->layouts[i].variant;
        if (i<xkbConf->layouts.count()-1)
        {
            layout+=",";
            variant+=",";
        }
    }
    option = xkbConf->options.join(",");
    qDebug()<<"Set layout : "<<layout;
    if (!(model.isEmpty() || model.isNull()) && !(layout.isEmpty() || layout.isNull()))
    {
        QString args = "-model "+model+" -layout "+layout;
        QString tmp = variant;
        if (!variant.isNull() && !variant.isEmpty() && !tmp.remove(",").isEmpty())
          args +=" -variant "+variant;
        if (!option.isNull() && !option.isEmpty())
        args +=" -option "+option;
        qDebug() << "XKB args " << args;
        if (setKeyLayout(args) == QProcess::CrashExit)
        {
            qDebug() << "XKB isn`t set";
            qDebug() << "------------------------------";
            return;
        }
        else
            qDebug() << "XKB  set";
    }
}

void  AXKB::actionsActivate(QAction * action)
{
    QString cmd = action->data().toString();
    qDebug()<<cmd;
    if(cmd == "configure")
        configure();
    else if (cmd == "about_qt")
    {
        About dlg;
        dlg.aboutQt(tr("Antico XKB"));
        dlg.exec();
    }
    else if (cmd == "about_axkb")
    {
        QString about = tr("Antico XKB.\n Version: %1.\n Gui tools for configure  xkb extention of x server.\n 2009(c) Fedor Chelbarakh" ).arg("0.4.1");
        About dlg;
        dlg.about (tr("Antico XKB"),about);
        dlg.exec();

     }
    else if (cmd == "exit")
        exit();
    else
      keys->setGroupNo(groupeName.indexOf(cmd));
}

void  AXKB::configure()
{
    AnticoXKBconf* xkbconf = new AnticoXKBconf();
   connect(xkbconf,SIGNAL(saveConfig()),SLOT(reconfigure()));
   xkbconf->exec();
}

bool AXKB::load_rules()
{
    QString x11dir = X11tools::findX11Dir();
    if ( x11dir.isNull() || x11dir.isEmpty())
        return false;
     QString rulesFile = X11tools::findXkbRulesFile(x11dir,QX11Info::display());
     if ( rulesFile.isNull() || rulesFile.isEmpty())
        return false;

    rule = X11tools::loadRules(rulesFile,false);
    if (rule==NULL)
        return false;

   return true;

}

void AXKB::setLanguageMap(int index)
{
    int cur_lang;
    Window curent_window=getActiveWindowId();
    QString current_app=getActiveWindowAppName();
   clearLangMap();
   cur_lang= app_lang.value(current_app,-1);
    if (cur_lang<0)
            app_lang.insert(current_app,index);
    else
         if (index!=cur_lang)
        {
            app_lang.remove(current_app);
            app_lang.insert(current_app,index);
         }
  cur_lang=window_lang.value(curent_window,-1);
    if (cur_lang<0)
            window_lang.insert(curent_window,index);
    else
        if (index!=cur_lang)
        {
            window_lang.remove(curent_window);
            window_lang.insert(curent_window,index);
         }
    if (app_window.value(curent_window,"").isEmpty())
        app_window.insert(curent_window,current_app);

}

void AXKB::checkLayoutChenge()
{
    if   (xkbConf->switching  ==GLOBAL_LAYOUT) return;

    int cur_lang;
    Window curent_window=getActiveWindowId();
    if (active_window == curent_window) return;
    active_window = curent_window;

    QString current_app=getActiveWindowAppName();
      qDebug()<<"Switch stat"<<xkbConf->switching ;
 if   (xkbConf->switching  ==APP_LAYOUT)
     cur_lang=app_lang.value(current_app,-1);
 if   (xkbConf->switching  ==WIN_LAYOUT)
     cur_lang=window_lang.value(curent_window,-1);

  qDebug()<<"Selected lang : "<<cur_lang;
  qDebug()<<"Curent window :"<<curent_window;
  if (cur_lang<0)
  {
      setLanguageMap(keys->getGroupNo());
      return;
  }
   keys->setGroupNo(cur_lang);
}

void AXKB::clearLangMap()
{

    QHashIterator<Window, int> i(window_lang);
    while (i.hasNext())
    {
     i.next();
      XWindowAttributes *window_attributes_return;
      window_attributes_return = (XWindowAttributes *) malloc(255);
      Status  stat = XGetWindowAttributes(QX11Info::display(), i.key(), window_attributes_return);
     if (stat== BadDrawable || stat==BadWindow)
      {
         window_lang.remove(i.key());
         app_lang.remove(app_window.value(i.key()));
        app_window.remove(i.key());
     }

    }
}
